@@ -1,6 +1,6 @@
 # models.py
 #
-# Copyright (C) 2020-2023 Fabio Comuni, et al.
+# Copyright 2020-2022 Fabio Comuni, et al.
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -25,16 +25,24 @@ from . import local
 from .settings import Settings
 from .fetcher import Fetcher
 
+
+class ObjectNotFound(Exception):
+    pass
+
+
 ## models
-def _build_filter(day=None, room=None, track=None, event=None, **kwargs):
+def _build_filter(table="", day=None, room=None, track=None, event=None, **kwargs):
         filterdata = []
         wherestm = []
 
+        if table != "":
+            table = f"{table}."
+
         def __f(col, value):
             if value is None:
-                wherestm.append("{} is null".format(col))
+                wherestm.append(f"{table}{col} is null")
             else:
-                wherestm.append("{} = ?".format(col))
+                wherestm.append(f"{table}{col} = ?")
                 filterdata.append(value)
 
         if day is not None:
@@ -95,12 +103,15 @@ class Conference:
             return None
         return cls()._init(ss[0])
 
+    def has_cache(self):
+        """Check if there is local db for this conference"""
+        return local.existsdb(self.url)
+
     def get_meta(self):
         """Return metadata from db. This will close any opened db connection."""
         local.close()
         local.opendb(self.url, False)
         return Meta()
-        local.close()
 
     def to_json(self):
         """Export conference meta as json for user menu"""
@@ -115,27 +126,27 @@ class Meta:
     """Conference metadata from "meta" table in db"""
     def __init__(self):
         _c = local.getDb().execute("SELECT key, value FROM meta")
-        _d = [(row['key'], row['value']) for row in _c]
-        self._data = dict(_d)
+        _d = [(row['key'], self._fmtd_value(row['key'], row['value'])) for row in _c]
+        self._data = { 'last_update': 0 }
+        self._data.update(dict(_d))
 
-    def _fmtd_value(self, k):
-        v = self._data.get(k, None)
+    def _fmtd_value(self, k, v):
         if k == "start" or k == "end":
             if v is not None and v != "":
                 if ":" in v:
                     v = datetime.datetime.strptime(v, "%Y-%m-%d %H:%M:%S")
                 else:
-                    v = datetime.datetime.strptime(v, "%Y-%m-%d")
+                    v = datetime.date(*[int(s) for s in v.split("-")])
             else:
                 v = None
         if k == "last_update":
-            v = 0 if v is None else float(v)
+            v = float(v)
         return v
 
     def __getattr__(self, name):
         if name.startswith("_"):
             return super().__getattr__(name)
-        return self._fmtd_value(name)
+        return self._data.get(name)
 
     def __setattr__(self, name, value):
         if name.startswith("_"):
@@ -305,10 +316,19 @@ class Link:
         return '<a href="{}">{}</a>'.format(self.href, self.name)
 
 
+class DbUpdateWatcherObj(GObject.GObject):
+    @GObject.Signal
+    def update(self):
+        ...
+
+
+dbUpdateWatcher = DbUpdateWatcherObj()
+
+
 class Event(GObject.GObject):
-    __gsignals__ = {
-        'update': (GObject.SIGNAL_RUN_LAST, GObject.TYPE_NONE, ())
-    }
+    @GObject.Signal
+    def update(self):
+        ...
 
     def _init(self, data):
         self.id = data['id']
@@ -332,13 +352,14 @@ class Event(GObject.GObject):
         local.getDb().commit()
         self.starred = starred
         self.emit('update')
+        dbUpdateWatcher.emit('update')
 
     def set_notified(self, notified):
         local.getDb().execute("UPDATE events SET notified=? WHERE id=?",(notified,self.id,))
         local.getDb().commit()
         self.notified = notified
         self.emit('update')
-
+        dbUpdateWatcher.emit('update')
 
     def start_in_tz(self):
         return self._dt_in_tz(self.start)
@@ -381,16 +402,31 @@ class Event(GObject.GObject):
 
     @classmethod
     def all(cls):
+        row = local.getDb().execute("SELECT count(id) as c FROM events", filterdata).fetchone()
+        yield 0 if row is None else row['c']
+
         for row in local.getDb().execute("SELECT * FROM events"):
             yield cls()._init(row)
 
+    @classmethod
+    def by_id(cls, id):
+        row = local.getDb().execute("SELECT * FROM events WHERE id = ?", (id,)).fetchone()
+        if row is None:
+            raise ObjectNotFound("<{} #{}>".format(cls.__name__, id))
+        return cls()._init(row)
+
     @classmethod
     def filter(cls, day:Day = None, room:Room = None, track:Track = None, **kwargs):
-        filterdata, wherestm = _build_filter(day=day, room=room, track=track, **kwargs)
+        filterdata, wherestm = _build_filter("events", day=day, room=room, track=track, **kwargs)
 
         query = """SELECT * FROM events
                     WHERE {}
                     ORDER BY start, room""".format(" AND ".join(wherestm))
+
+        resultcountquery = query.replace("SELECT *", "SELECT count(events.id) as c")
+        row = local.getDb().execute(resultcountquery, filterdata).fetchone()
+        yield 0 if row is None else row['c']
+
         for row in local.getDb().execute(query, filterdata):
             yield cls()._init(row)
 
@@ -408,20 +444,49 @@ class Event(GObject.GObject):
             yield cls()._init(row)
 
     @classmethod
-    def search(cls, term, starred=False):
+    def search(cls, term, starred=False, order_by=None, **filters):
         """Search events for 'term'"""
-
-        starfilter = ""
         if starred:
-            starfilter = "events.starred = 1 AND"
+            filters["starred"] = 1
+        filterdata, wherestm = _build_filter("events", **filters)
 
-        query = """SELECT events.* FROM fts_event
-                    JOIN events ON events.id = fts_event.event_id
-                    WHERE {} fts_event = ?
-                    GROUP BY events.id
-                    ORDER BY rank""".format(starfilter)
+        if term == "" and len(wherestm) == 0:
+            yield 0
+            return
 
-        for row in local.getDb().execute(query, (term,)):
+        if term != "":
+            order = ['rank']
+            if order_by:
+                order = [order_by, 'rank']
+
+            wherestm.append("fts_event = ?")
+            filterdata.append(term)
+
+            query = """SELECT events.* FROM fts_event
+                        JOIN events ON events.id = fts_event.event_id
+                        WHERE {}
+                        GROUP BY events.id
+                        ORDER BY {}"""
+        else:
+            order = ['start', 'room']
+            if order_by:
+                order = [order_by]
+            query = """SELECT events.* FROM events
+                        WHERE {}
+                        ORDER BY {}"""
+
+        query = query.format(
+            " AND ".join(wherestm),
+            ", ".join(order)
+        )
+
+        resultcountquery = query.replace("SELECT events.*", "SELECT count(events.id) as c")
+        row = local.getDb().execute(resultcountquery, filterdata).fetchone()
+        yield 0 if row is None else row['c']
+
+
+        results = local.getDb().execute(query, filterdata)
+        for row in results:
             yield cls()._init(row)
 
 
