@@ -1,6 +1,6 @@
 # settings.py
 #
-# Copyright (C) 2020-2023 Fabio Comuni, et al.
+# Copyright 2020-2022 Fabio Comuni, et al.
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -15,30 +15,31 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
+from enum import IntEnum
+
 from gi.repository import Gio
-from gi.repository import Handy
+from gi.repository import Gtk
 from dateutil import tz
 
+
+
 # in secs
 CACHE_NOCACHE = 0
 CACHE_ONE_HOUR = 60 * 60
 CACHE_ONE_DAY = 24 * CACHE_ONE_HOUR
 CACHE_ONE_WEEK = 7 * CACHE_ONE_DAY
 
-class CacheDuration:
-    NOCACHE = 0
-    ONEHOUR = 1
-    ONEDAY = 2
-    ONEWEEK = 3
+
+class CacheDuration(IntEnum):
+    NO_CACHE = 0
+    ONE_HOUR = 1
+    ONE_DAY = 2
+    ONE_WEEK = 3
 
     @classmethod
     def as_model(cls):
-        m = Gio.ListStore.new(Handy.ValueObject)
-        m.insert(0, Handy.ValueObject.new(_("No cache")))
-        m.insert(1, Handy.ValueObject.new(_("One hour")))
-        m.insert(2, Handy.ValueObject.new(_("One day")))
-        m.insert(3, Handy.ValueObject.new(_("One week")))
-        return m
+        labels = [ n.name.replace("_", " ").title() for n in cls ]
+        return Gtk.StringList.new(labels)
 
     @classmethod
     def get_duration(cls, enum_value):
@@ -49,7 +50,6 @@ class CacheDuration:
             CACHE_ONE_WEEK
         ][enum_value]
 
-
 class Settings(Gio.Settings):
     _instance = None
 
@@ -81,4 +81,11 @@ class Settings(Gio.Settings):
 
     def get_timezone(self):
         return tz.tzlocal()
-        
+
+    def get_size(self):
+        return (self.get_int('window-width'), self.get_int('window-height'))
+
+    def set_size(self, size):
+        self.set_int('window-width', size[0])
+        self.set_int('window-height', size[1])
+
